/* minimal program to open & clear a window */
#include <cmath>
#include <iostream>
#include <GL/glew.h>
#include <GL/freeglut.h>
#include <GL/freeglut_ext.h>
#include "moglclass.h"
#include "shaders.h"
#include "vertexBufferObject.h"
#include "geometryPositionColor.h"
#include "simpleCar.h"
#include "simpleRobot.h"
#include "simpleSolar.h"
#include "simpleMobile.h"

using namespace moglclass;

#define NUMSHADERS 2
Shader shShaders[NUMSHADERS];
Program spMain;

int startTime = 0;

Cube * cube[4];

bool g_mousemove = false;
bool g_wiremode = false;
bool g_animationmode = false;
int g_transformationmode = 0;

float g_fovy = 45.0f;
float g_aspect = 1.0f;
float g_zNear = 1.0f;
float g_zFar = 100.f;

glm::vec3 g_eye(0, 0, 10);
glm::vec3 g_at(0, 0, 0);
glm::vec3 g_up(0, 1, 0);

glm::mat4 World(1.0f);
glm::mat4 View(1.0f);
glm::mat4 Projection(1.0f);

float g_theta = 0.0f;

float g_xangle = 0.0f;    // => 30.0f
float g_yangle = 0.0f;    // => 60.0f
float g_zangle = 0.0f;    // => 45.0f
enum AngleMode { None, Yaw, Pitch, Roll };
AngleMode g_anglemode = AngleMode::None;

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
void mprint(glm::mat4& Mat)
{
	printf("\n %f %f %f %f\n %f %f %f %f\n %f %f %f %f\n %f %f %f %f\n\n",
		Mat[0][0], Mat[1][0], Mat[2][0], Mat[3][0],
		Mat[0][1], Mat[1][1], Mat[2][1], Mat[3][1],
		Mat[0][2], Mat[1][2], Mat[2][2], Mat[3][2],
		Mat[0][3], Mat[1][3], Mat[2][3], Mat[3][3]);
}
// set geometry
void setData()
{
}

void init( void )
{
    // Load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("simple4.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("simple4.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();

	setData();
	
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
}

void display( void )
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); // clear the window

	// set drawmode
	if (g_wiremode) 
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  // draw wireframe 
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	// draw filled 

	// MVP matrix
	Projection = glm::perspective(g_fovy, g_aspect, g_zNear, g_zFar);
	View = glm::lookAt(g_eye, g_at, g_up);
	spMain.useProgram();
	spMain.setUniform("gProjection", Projection);
	spMain.setUniform("gView", View);	

	glm::mat4 Ry = glm::rotate(glm::mat4(1.0f), g_yangle, glm::vec3(0.0f, 1.0f, 0.0f));
	glm::mat4 Rx = glm::rotate(glm::mat4(1.0f), g_xangle, glm::vec3(1.0f, 0.0f, 0.0f));
	glm::mat4 Rz = glm::rotate(glm::mat4(1.0f), g_zangle, glm::vec3(0.0f, 0.0f, 1.0f));
	glm::mat4 R = Rz * Rx * Ry; // world coordinate system rotation
	//printf("R =\n"); mprint(R);
	glm::mat4 YPR = glm::yawPitchRoll(g_yangle, g_xangle, g_zangle); // Creates a 3D 4 * 4 homogeneous rotation matrix from euler angles (YXZ).  local coordinate system rotation
	//printf("YPR =\n"); mprint(YPR);

	{
	glVertexAttrib3f((GLuint)1, 1, 0, 0); // set constant color attribute (red)
	World = glm::mat4(1.0f);
	spMain.setUniform("gModel", World);
	glutWireTeapot(1.0);
	}

	{
	glVertexAttrib3f((GLuint)1, 0, 1, 0); // set constant color attribute (green)
	World = glm::translate(glm::mat4(1.0f), glm::vec3(-3.0f, 0.0f, 0.0f)) * R;
	spMain.setUniform("gModel", World);
	glutWireTeapot(1.0);
	}

	{
	glVertexAttrib3f((GLuint)1, 0, 0, 1); // set constant color attribute (blue)
	World = glm::translate(glm::mat4(1.0f), glm::vec3(3.0f, 0.0f, 0.0f)) * YPR ;
	spMain.setUniform("gModel", World);
	glutWireTeapot(1.0);
	}

	glBindVertexArray(0);
	glDisableVertexAttribArray(0);

	glutSwapBuffers();
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0; 
	static int elapsedTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;
	prevTime = currentTime;
	elapsedTime = currentTime - startTime;

	elapsedTime = currentTime - startTime;
	glutPostRedisplay();
}

void keyboard( unsigned char key, int x, int y )
{
	switch ( key ) {
		case 27:
			exit( EXIT_SUCCESS );
			break;
		case 'w':
			g_wiremode = !g_wiremode;
			break;
		case 'a':
			g_animationmode = !g_animationmode;
			break;
		case 't':
			g_transformationmode = (g_transformationmode + 1) % 4;
			printf("g_transformationmode=%d\n", g_transformationmode);
			break;
		case ' ':
			g_anglemode = (AngleMode)((g_anglemode + 1) % 4);
			printf("g_anglemode=%d\n", g_anglemode);
			if (g_anglemode == AngleMode::None)
			{
				g_xangle = g_yangle = g_zangle = 0.0f;
			}
			else if (g_anglemode == AngleMode::Yaw)
			{
				g_yangle = DEGTORAD(60.0f);
			}
			else if (g_anglemode == AngleMode::Pitch)
			{
				g_xangle = DEGTORAD(30.0f);
			}
			else if (g_anglemode == AngleMode::Roll)
			{
				g_zangle = DEGTORAD(45.0f);
			}
			break;
	}
	glutPostRedisplay();
}

void specialkey(int key, int x, int y )
{
	switch (key)  
	{    
		case GLUT_KEY_LEFT:
			break;  
		case GLUT_KEY_RIGHT:    
			break;  
	}
	glutPostRedisplay();
}

void mouse(int button, int state, int mx, int my)
{
	if (button==GLUT_RIGHT_BUTTON && state==GLUT_DOWN)
		exit(0);
	if (button==GLUT_LEFT_BUTTON && state==GLUT_DOWN)
		g_mousemove = true;
	else if (button==GLUT_LEFT_BUTTON && state==GLUT_UP)
		g_mousemove = false;
}

void motion(int mx, int my)
{
	int w = glutGet(GLUT_WINDOW_WIDTH);
	int h = glutGet(GLUT_WINDOW_HEIGHT);

	// 0~600(x+ right) x 0~600(y+ down) => -5~5(x+ right) x -5~5(y+ up)
	float x = (float)  10.0f * (mx - w*0.5f) / w;
	float y = (float) 10.0f * (h*0.5f - my) / h;

	if (g_mousemove)
	{
		//printf("mx=%d my=%d x=%f y=%f\n", mx, my, x, y);
		//geoList[g_geometrymode]->setPosition(glm::vec3(x, y, 0));
	}

	glutPostRedisplay();
}

void reshape(int w, int h)
{
	g_aspect = (float) (w/h);

    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glutPostRedisplay();
}

void cleanup()
{		 
	spMain.deleteProgram();
}

int main( int argc, char **argv )
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA  | GLUT_DEPTH);
	glutInitWindowSize( 600, 600 );
	glutCreateWindow( "Geometry Primitives Transformation" );

	glewInit();
	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutSpecialFunc( specialkey );
	glutMouseFunc( mouse ); 
	glutMotionFunc( motion ); 
	glutIdleFunc( update );
	glutReshapeFunc( reshape );
	glutMainLoop();

	cleanup();
	return 0;
}